package com.xxl.job.executor.registry;

import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.IJobHandler;
import groovy.util.logging.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;


//@Component
@Slf4j
public class JobHandlerAutoRegister implements ApplicationContextAware {

    private ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
        initJob();
    }

    private void initJob() {
        Map<String, Object> serviceBeanMap = applicationContext.getBeansWithAnnotation(XXLJobHandler.class);

        for (Object serviceBean : serviceBeanMap.values()) {
            if (serviceBean instanceof IJobHandler) {
                XXLJobHandler xxlJobHandler = serviceBean.getClass().getAnnotation(XXLJobHandler.class);
                if (xxlJobHandler != null) {
                    MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
                    paramMap.add("jobGroup", xxlJobHandler.jobGroup() + "");
                    paramMap.add("jobDesc", xxlJobHandler.jobDesc());
                    paramMap.add("executorRouteStrategy", xxlJobHandler.executorRouteStrategy());
                    paramMap.add("jobCron", xxlJobHandler.jobCron());
                    paramMap.add("glueType", xxlJobHandler.glueType());
                    paramMap.add("executorHandler", xxlJobHandler.executorHandler());
                    paramMap.add("executorParam", xxlJobHandler.executorParam());
                    paramMap.add("childJobId", xxlJobHandler.childJobId());
                    paramMap.add("executorTimeout", xxlJobHandler.executorTimeout() + "");
                    paramMap.add("executorBlockStrategy", xxlJobHandler.executorBlockStrategy());
                    paramMap.add("executorFailRetryCount", xxlJobHandler.executorFailRetryCount() + "");
                    paramMap.add("alarmEmail", xxlJobHandler.alarmEmail());
                    paramMap.add("author", xxlJobHandler.author());

                    HttpHeaders headers = new HttpHeaders();
                    headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

                    HttpEntity requestEntity = new HttpEntity(paramMap, headers);

                    RestTemplate restTemplate = applicationContext.getBean(RestTemplate.class);
                    ResponseEntity<ReturnT> returnTResponseEntity = restTemplate.postForEntity("http://localhost:8333/xxl-job-admin/jobinfo/add", requestEntity, ReturnT.class);
                    System.out.println(returnTResponseEntity.getBody().getCode());
                }
            }
        }
    }


}
